# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Module for the AltitudeDict class."""

from __future__ import annotations

from importlib.resources import files
import sqlite3

from .lunar_feature import LunarFeature
from .moon_info import MoonInfo

__all__ = ["AltitudeDict"]


class AltitudeDict(dict[str, float]):
    """Dictionary for the Lunar II features requiring solar altitude."""

    def load(self, moon_info: MoonInfo) -> None:
        """Provide solar altitude for the Lunar II features.

        Parameters
        ----------
        moon_info : :class:`pylunar.MoonInfo`
            Instance of the Lunar information class.
        """
        features = ["Byrgius A", "Proclus", "Rupes Recta", "Tycho"]
        dbname = str(files("pylunar.data").joinpath("lunar.db"))
        conn = sqlite3.connect(dbname)
        cur = conn.cursor()

        sql = f"select * from Features where Name in {str(tuple(features))}"
        cur.execute(sql)

        feature_list = []
        for row in cur:
            feature_list.append(LunarFeature.from_row(row))

        for feature in sorted(feature_list, key=lambda x: x.name):
            self[feature.name] = moon_info.solar_altitude(feature)

        cur.close()



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Module for holding types."""

from __future__ import annotations

from typing import TypeAlias

DateTimeTuple: TypeAlias = tuple[int, ...] | tuple[int, int, int, int, int, float]
MoonPhases: TypeAlias = list[tuple[str, DateTimeTuple | str]]
DmsCoordinate: TypeAlias = tuple[int, int, int]
Range: TypeAlias = tuple[float, float]
LunarFeatureList: TypeAlias = tuple[str, float, float, float, float, float, str, str, str, str, str | None]
FeatureRow: TypeAlias = tuple[int, str, float, float, float, float, float, str, str, str, str, str]



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Tests for AltitudeDict class."""

from pylunar import AltitudeDict, MoonInfo


class TestAltitudeDict:
    def setup_class(self) -> None:
        self.ad = AltitudeDict()

    def test_basic_information_after_creation(self) -> None:
        assert len(self.ad) == 0

    def test_information_after_load(self) -> None:
        location = ((35, 58, 10), (-84, 19, 0))
        mi = MoonInfo(location[0], location[1])
        mi.update((2013, 10, 12, 18, 0, 0))

        self.ad.load(mi)
        assert len(self.ad) == 4
        assert list(self.ad.keys()) == ["Byrgius A", "Proclus", "Rupes Recta", "Tycho"]
        assert self.ad["Rupes Recta"] == 1.3038753306292297
